#! /usr/local/bin/perl

my $en_file = "usecode.en.txt";
my $ja_file = "usecode.ja.txt";

sub usage() {
	print "使用方法: calc_rate.pl [オプション]\n";
	print "機    能: 翻訳率を表示します\n";
	print "オプション:\n";
	print "    -j|--ja-text ファイル      指定したファイルから日本語テキストを読み込みます\n";
	print "    -e|--en-text ファイル      指定したファイルから英語テキストを読み込みます\n";
	print "    -?|--help                  使用方法を表示します\n";
}

sub read_text($) {
	my $filename = shift @_;
	my %hash;
	my $line = 0;

	open(my $FILE, '<',  $filename) or die "ERROR: can't open $filename: $!";

	while (<$FILE>) {
		$line++;

		s/[\r\n]+$//;
		next if (/^$/);
		next if (/^[#;]/);

		my ($func_id, $label_name, $font_type, $text) = split('\t', $_, 4);

		if ( $text !~ m/^\"(.*)\"$/ ) {
			printf(STDERR "SYNTAX ERROR: %s line %d\n", $filename, $line);
			exit 1;
		}

		$hash{$label_name} = $font_type . "\t" . $text;
	}
	close($FILE);

	return %hash;
}

while (my $var = shift @ARGV) {
	if ($no_more_options == 0 and $var =~ m/^-/) {
		if ($var eq "--") {
			$no_more_options = 1;
		} elsif ($var eq "-?" or $var eq "--help") {
			usage();
			exit 1;
		} elsif ($var eq "-j" or $var eq "--ja-text") {
			if ($#ARGV < 0) {
				printf(STDERR "ERROR: %s without filename\n", $var);
				exit 1;
			}
			$ja_file = shift @ARGV;
		} elsif ($var eq "-e" or $var eq "--en-text") {
			if ($#ARGV < 0) {
				printf(STDERR "ERROR: %s without filename\n", $var);
				exit 1;
			}
			$en_file = shift @ARGV;
		} else {
			printf(STDERR "ERROR: unknown option: %s\n", $var);
			exit 1;
		}
	} else {
		printf(STDERR "ERROR: unknown parameter: %s\n", $var);
		exit 1;
	}
}

my %en_text = read_text($en_file);
my %ja_text = read_text($ja_file);

sub count_text($) {
	my $target = shift @_;
	my $en_count = 0;
	my $ja_count = 0;

	foreach my $label (keys %en_text) {
		my ($font_type, $text) = split("\t", $en_text{$label}, 2);
		next if ($font_type ne $target);

		$en_count++;
		if (defined $ja_text{$label} and $en_text{$label} ne $ja_text{$label}) {
			$ja_count++;
		}
	}

	return ($en_count, $ja_count);
}

my $en_total = 0;
my $ja_total = 0;

print "種別                        英語          日本語          翻訳率\n";
print "-" x 64, "\n";

foreach my $font_type ( "CHOICE", "CONVERSATION", "BOOK", "SCROLL", "SIGN", "TOMB", "PLAQUE", "S.BOOK", "S.SCROLL", "S.PLAQUE", "TEXT" ) {
	my ($en, $ja) = count_text($font_type);

	if (0 < $en) {
		printf("%-16s\t%8.1d\t%8.1d\t%7.2f%%\n", $font_type, $en, $ja, $ja * 100 / $en);
		$en_total += $en;
		$ja_total += $ja;
	}
}

print "-" x 64, "\n";
printf("%-16s\t%8.1d\t%8.1d\t", "TOTAL", $en_total, $ja_total);
if (0 < $en_total) {
	printf("%7.2f%%", $ja_total * 100 / $en_total);
}
printf("\n");

# EOF
