#! /usr/local/bin/perl

my %en_names;
my $text_file = "usecode.ja.txt";
my $line = 0;
my $num_params = 0;

sub usage() {
	print "使用方法: list_en_names.pl [オプション] [usecode.ja.txt]\n";
	print "機    能: usecode.ja.txtから英単語を探し表示します\n";
	print "オプション:\n";
	print "    -?|--help                  使用方法を表示します\n";
	print "\n";
	print "{と}で囲まれた範囲に英語がある場合は、{と}で囲まれた全体を表示します\n";
	print "\n";
	print "既知の不具合\n";
	print "    ふるい分けがあまり上手くできておらず日本語と英語が混在している箇所の\n";
	print "    英文全体を表示してしまいます。\n";
}

while (my $var = shift @ARGV) {
	if ($no_more_options == 0 and $var =~ m/^-/) {
		if ($var eq "--") {
			$no_more_options = 1;
		} elsif ($var eq "-?" or $var eq "--help") {
			usage();
			exit 1;
		} else {
			printf(STDERR "ERROR: unknown option: %s\n", $var);
			exit 1;
		}
	} else {
		if ($num_params == 0) {
			$text_file = $var;
		} else {
			printf(STDERR "ERROR: unknown parameter: %s\n", $var);
			exit 1;
		}
		$num_params++;
	}
}

open(my $FILE, '<', $text_file) or die "ERROR: can't open $text_file: $!\n";
while (<$FILE>) {
	$line++;

	s/[\r\n]+$//;
	next if (/^$/);
	next if (/^[#;]/);

	my ($func_id, $label_name, $font_type, $text) = split('\t', $_, 4);

	if ( $text =~ m/^\"(.*)\"$/ ) {
		$text = $1;
	} else {
		printf(STDERR "SYNTAX ERROR: %s line %d\n", $text_file, $line);
		exit 1;
	}

	foreach ( $text =~ m/\{([^ -z]*[ -z]+[^ -z]*)\}/ ) {
		my $found = $1;
		last if ($text eq $found);

		if ( defined $en_names{$found} ) {
			$en_names{$found}++;
		} else {
			$en_names{$found} = 1;
		}
	}
	foreach ( $text =~ m/([ -z~]+)/ ) {
		my $found = $1;
		last if ($text eq $found);
		$found =~ s/^([@!?~ ^,.:]|\\[trn\\\"])+//;
		$found =~ s/([@ ,.!?~]|\\[trn\\\"])+$//;
		$found =~ s/\($//;
		$found =~ s/^\)//;
		$found =~ s/^[a-zA-Z]:[ \t]*//;
		$found =~ s/^\((.*)\)$/$1/;
		next if ($found eq "");
		next if ($found =~ m/^[^a-zA-Z]*$/);

		if ( defined $en_names{$found} ) {
			$en_names{$found}++;
		} else {
			$en_names{$found} = 1;
		}
	}
}
close($FILE);

foreach $name ( sort keys %en_names ) {
	print $name, "\n";
}

# EOF
