#! /usr/local/bin/perl

my @text_files;
my %merged;

my $opt_sort_mtime = 0;

sub usage() {
	print "使用方法: merge_text.pl [オプション] usecode.ja.txt ...\n";
	print "機    能: 複数のusecode.ja.txtを一つに纏めます\n";
	print "          重複するテキストは後から読み込んだものに置き換えます\n";
	print "オプション:\n";
	print "    -t|--sort-mtime             ファイルを更新日付順に読み込みます\n";
	print "                                また、標準エラー出力に更新日付とファイル名の\n";
	print "                                一覧を表示します\n";
	print "    -?|--help                   使用方法を表示します\n";
}

while (my $var = shift @ARGV) {
	if ($no_more_options == 0 and $var =~ m/^-/) {
		if ($var eq "--") {
			$no_more_options = 1;
		} elsif ($var eq "-?" or $var eq "--help") {
			usage();
			exit 1;
		} elsif ($var eq "-t" or $var eq "--sort-mtime") {
			$opt_sort_mtime = 1;
		} else {
			printf(STDERR "ERROR: unknown option: %s\n", $var);
			exit 1;
		}
	} else {
		push @text_files, $var;
	}
}

if ($#text_files < 0) {
	usage();
	exit 1;
}

if ($opt_sort_mtime) {
	my %mtime_files;
	foreach my $text_file ( @text_files ) {
		my @st = stat($text_file);
		if ($#st < 0) {
			printf(STDERR "ERROR: can't stat %s\n", $text_file);
			exit 1;
		}
		my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($st[9]);
		my $basename = $text_file;
		if ($basename =~ m#^.*[/\\]([^/\\]+)$#) {
			$basename = $1;
		}
		my $key = sprintf("%04d-%02d-%02d %02d:%02d:%02d %s", $year + 1900, $mon + 1, $mday, $hour, $min, $sec, $basename);
		$mtime_files{$key} = $text_file;
	}

	@text_files = ();
	foreach my $key (sort keys %mtime_files) {
		printf(STDERR "%s\t%s\n", substr($key, 0, 19), $mtime_files{$key});
		push @text_files, $mtime_files{$key};
	}
}

foreach my $text_file ( @text_files ) {
	my $line = 0;

	open(my $FILE, '<', $text_file) or die "can't open $text_file: $!\n";
	while (<$FILE>) {
		$line++;

		s/[\r\n]+$//;
		next if (/^$/);
		next if (/^[#;]/);

		my ($func_id, $label_name, $font_type, $text) = split('\t', $_, 4);

		if ( $text !~ m/^\".*\"$/ ) {
			printf(STDERR "SYNTAX ERROR: %s line %d\n", $filename, $line);
			exit 1;
		}

		$merged{$func_id . "\t" . $label_name} = $_;
	}
	close($FILE);
}

print ";TAB SEPALATED FILE\n";
print ";func_id\tlabel_name\tfont_no\tquoted_string\n";

foreach $index (sort keys %merged) {
	print $merged{$index} . "\n";
}

# EOF
