#! /usr/local/bin/perl

my $src_file = "usecode.src";
my $text_file = "usecode.ja.txt";
my $num_params = 0;

my $opt_no_label = 0;

sub usage() {
	print "使用方法: replace_src_text.pl [オプション] [usecode.src]\n";
	print "機    能: usecode.src中のテキストをusecode.ja.txtのものに置換して表示します\n";
	print "オプション:\n";
	print "    -t|--text ファイル          指定したファイルからテキストを読み込みます\n";
	print "       --no-label               テキストの前にあるラベル部分を除去します\n";
	print "    -?|--help                   使用方法を表示します\n";
	print "\n";
	print "既知の不具合:\n";
	print "    テキスト定義部分は長いテキストでも折り返ししていません\n";
}

sub read_text($) {
	my $filename = shift @_;
	my %hash;
	my $line = 0;

	open(my $FILE, '<',  $filename) or die "ERROR: can't open $filename: $!";

	while (<$FILE>) {
		$line++;

		s/[\r\n]+$//;
		next if (/^$/);
		next if (/^[#;]/);

		my ($func_id, $label_name, $font_type, $text) = split('\t', $_, 4);

		if ( $text !~ m/^\".*\"$/ ) {
			printf(STDERR "SYNTAX ERROR: %s line %d\n", $filename, $line);
			exit 1;
		}

		$hash{$label_name} = $text;
	}
	close($FILE);

	return %hash;
}

sub replace_text($%) {
	my $file = shift @_;
	my %text = @_;

	my $def_cont = 0;
	my $line = 0;

	open($FILE, '<', $file) or die "ERROR: can't open $file: $!";
	while(<$FILE>) {
		$line++;

		if ($def_cont != 0) {
			if (m/^[ \t]*\".*\"(;| \\)$/) {
				if ($1 eq ";") {
					$def_cont = 0;
				}
			} else {
				printf(STDERR "ERROR: syntax error: %s line %d\n", $file, $line);
				exit 1;
			}
			next;
		}

		if (m/^[ \t]*(L[0-9A-F]{4}_D[0-9]{4})[ \t]*=[ \t]*\".*\"(;| \\)$/) {
			if (defined $text{$1}) {
				if ($2 ne ";") {
					$def_cont = 1;
				}
				printf("\t%s = %s;\n", $1, $text{$1});
			} else {
				print;
			}
			next;
		}

		my %found;
		while (m/(L[0-9A-F]{4}_D[0-9]{4}):(\"(?:[^\\\"]|\\.)*\")/g) {
			$found{$1} = $2;
		}
		foreach my $label (keys %found) {
			my $pattern = $label . ":" . $found{$label};
			if (defined $text{$label}) {
				if ($opt_no_label) {
					s/\Q$pattern/$text{$label}/g;
				} else {
					s/\Q$pattern/$label:$text{$label}/g;
				}
			} else {
				if ($opt_no_label) {
					s/\Q$pattern/$found{$label}/g;
				}
			}
		}
		print;
	}
	close($FILE);
}

while (my $var = shift @ARGV) {
	if ($no_more_options == 0 and $var =~ m/^-/) {
		if ($var eq "--") {
			$no_more_options = 1;
		} elsif ($var eq "-?" or $var eq "--help") {
			usage();
			exit 1;
		} elsif ($var eq "-t" or $var eq "--text") {
			if ($#ARGV < 0) {
				printf(STDERR "ERROR: %s without filename\n", $var);
				exit 1;
			}
			$text_file = shift @ARGV;
		} elsif ($var eq "--no-label") {
			$opt_no_label = 1;
		} else {
			printf(STDERR "ERROR: unknown option: %s\n", $var);
			exit 1;
		}
	} else {
		if ($num_params == 0) {
			$src_file = $var;
		} else {
			printf(STDERR "ERROR: unknown parameter: %s\n", $var);
			exit 1;
		}
		$num_params++;
	}
}

my %text = read_text($text_file);
replace_text($src_file, %text);

# EOF
