#! /usr/local/bin/perl

my $en_file = "usecode.en.txt";
my $ja_file = "usecode.ja.txt";

my $no_more_options = 0;
my @words = ();

sub usage() {
	print "使用方法: search_text.pl [オプション] 検索文字列 ...\n";
	print "機    能: usecode.en.txtから検索文字列のいずれかを含むテキストを探し\n";
	print "          対応するusecode.ja.txtの行と共に表示します\n";
	print "オプション:\n";
	print "    -j|--ja-text ファイル      指定したファイルから日本語テキストを読み込みます\n";
	print "    -e|--en-text ファイル      指定したファイルから英語テキストを読み込みます\n";
	print "\n";
	print "    -?|--help                  使用方法を表示します\n";
	print "\n";
	print "検索文字列:\n";
	print "    検索文字列は複数指定できます\n";
	print "    英字の大文字小文字は区別しません\n";
	print "    エスケープしないでそのまま m/検索文字列/ でマッチさせているので正規表現が\n";
	print "    使えます\n";
	print "\n";
	print "コマンドパラメータに'--'があるとき、その後を全て検索文字列とみなします。\n";
	print "例えば'-one-'を検索したいときに\n";
	print "    search_text.pl -- -one-\n";
	print "とします。\n";
}

while (my $var = shift @ARGV) {
	if ($no_more_options == 0 and $var =~ m/^-/) {
		if ($var eq "--") {
			$no_more_options = 1;
		} elsif ($var eq "-?" or $var eq "--help") {
			usage();
			exit 1;
		} elsif ($var eq "-j" or $var eq "--ja-text") {
			if ($#ARGV < 0) {
				printf(STDERR "ERROR: %s without filename\n", $var);
				exit 1;
			}
			$ja_file = shift @ARGV;
		} elsif ($var eq "-e" or $var eq "--en-text") {
			if ($#ARGV < 0) {
				printf(STDERR "ERROR: %s without filename\n", $var);
				exit 1;
			}
			$en_file = shift @ARGV;
		} else {
			printf(STDERR "ERROR: unknown option: %s\n", $var);
			exit 1;
		}
	} else {
		push @words, $var;
	}
}

if ($#words < 0) {
	usage();
	exit 1;
}

sub read_text($) {
	my $filename = shift @_;
	my %hash;
	my $line = 0;

	open(my $FILE, '<',  $filename) or die "ERROR: can't open $filename: $!";

	while (<$FILE>) {
		$line++;

		s/[\r\n]+$//;
		next if (/^$/);
		next if (/^[#;]/);

		my ($func_id, $label_name, $font_type, $text) = split('\t', $_, 4);

		if ( $text =~ m/^\"(.*)\"$/ ) {
			$text = $1;
		} else {
			printf(STDERR "SYNTAX ERROR: %s line %d\n", $filename, $line);
			exit 1;
		}

		$hash{$label_name} = $text;
	}
	close($FILE);

	return %hash;
}

my %en_text = read_text($en_file);
my %ja_text = read_text($ja_file);

foreach my $key ( sort keys %ja_text ) {
	next if ($en_text{$key} eq $ja_text{$key});

	my $found = 0;
	foreach my $w ( @words ) {
		if ($en_text{$key} =~ m/$w/i) {
			$found = 1;
		}
		last if ($found != 0);
	}

	if ($found != 0) {
		printf("%s\t%s\n", $key, $en_text{$key});
		printf("%s\t%s\n", $key, $ja_text{$key});
		printf("\n");
	}
}

# EOF
